import React, { useState, useRef, useEffect } from "react";
import AuthAPI from "../common/apis/auth";
import { keyEvent } from "../common/lib/util";
import { KeyDetails, SignUpPayload, USER_TYPE } from "../common/types";

type OwnProps = {
  setAlert: (alert: string) => void;
  userType: USER_TYPE;
  setForm: (form: "sign-in" | "sign-up") => void;
};

const SignUp = ({ userType, setAlert, setForm }: OwnProps) => {
  const [is2FAenabled, setis2FAenabled] = useState<boolean>(false);
  const [credentials, setCredentials] = useState({
    name: "",
    email: "",
    password: "",
    confrimPassword: "",
    twoFAPassword: "",
  });

  const keydownArray = useRef<KeyDetails[][]>([]);
  const keyupArray = useRef<KeyDetails[][]>([]);
  const keyCount = useRef<number[]>([]);

  useEffect(() => {
    resetData();
  }, []);

  const resetData = () => {
    for (let i = 0; i < 3; i++) {
      keydownArray.current.push([]);
      keyupArray.current.push([]);
      keyCount.current.push(0);
    }
  };

  const onChange2FAconcent = () => setis2FAenabled(!is2FAenabled);
  const onChangeCredentials = (e: React.ChangeEvent<HTMLInputElement>) => {
    setCredentials({ ...credentials, [e.target.name]: e.target.value });
  };

  const onSumit = () => {
    if (credentials.password !== credentials.confrimPassword) {
      return setAlert("Passwords do not match");
    }

    if (!/^[\w-.]+@([\w-]+\.)+[\w-]{2,4}$/.test(credentials.email)) {
      return setAlert("Invalid email address");
    }

    const payload: SignUpPayload = {
      email: credentials.email,
      passwords: [
        credentials.password,
        credentials.confrimPassword,
        credentials.twoFAPassword,
      ],
      keydown: keydownArray.current,
      keyup: keyupArray.current,
      userType,
      candidate:
        userType === USER_TYPE.CANDIDATE
          ? { name: credentials.name, contacts: { email: credentials.email } }
          : undefined,
      organization:
        userType === USER_TYPE.ORGANIZATION
          ? { name: credentials.name, contacts: { email: credentials.email } }
          : undefined,
    };

    resetData();

    AuthAPI.signup(payload)
      .then((res: any) => {
        if (res.success) {
          setForm("sign-in");
          setAlert("Successfully signed up. Please login");
        }
      })
      .catch((error) => {
        setAlert(error.msg);
      });
  };

  const clearField = (index: number) => {
    if (index === 0) {
      setCredentials({ ...credentials, password: "" });
    } else if (index === 1) {
      setCredentials({ ...credentials, confrimPassword: "" });
    } else {
      setCredentials({ ...credentials, twoFAPassword: "" });
    }

    keydownArray.current[index] = [];
    keyupArray.current[index] = [];
    keyCount.current[index] = 0;
  };

  const onKeyDown = (e: any) => {
    const field = e.target.id;
    const index = Number(field.substr(field.length - 1));

    let details = keyEvent(e);
    if (!details) return;

    if (
      ["Backspace"].includes(details.code) ||
      ["Backspace"].includes(details.code)
    ) {
      return clearField(index);
    }

    if (
      ["Tab", "NumpadEnter"].includes(details.code) ||
      ["Tab"].includes(details.key)
    ) {
      details = {
        ...details,
        code: "Enter",
        key: "Enter",
      };
    }

    keydownArray.current[index].push(details);
    keyupArray.current[index].push({ ...details, time: null });
    keyCount.current[index]++;
  };

  const onKeyUp = (e: any) => {
    const field = e.target.id;
    const index = Number(field.substr(field.length - 1));

    let details = keyEvent(e);
    if (!details) return;
    if (!details.time) details.time = Date.now();

    if (
      ["Backspace"].includes(details.code) ||
      ["Backspace"].includes(details.code)
    ) {
      return clearField(index);
    }

    if (
      ["Tab", "NumpadEnter"].includes(details.code) ||
      ["Tab"].includes(details.key)
    ) {
      details = {
        ...details,
        code: "Enter",
        key: "Enter",
      };
    }

    let reqdUpKeystroke = keyupArray.current[index].find(
      (element) => element.code === details?.code && !element.time
    );

    if (reqdUpKeystroke) reqdUpKeystroke.time = details.time;
  };

  const disableSubmit =
    credentials.email === "" ||
    credentials.password === "" ||
    !is2FAenabled ||
    credentials.name === "";

  const nameLabel =
    userType === USER_TYPE.CANDIDATE ? "User name" : "Organization name";

  return (
    <>
      <h5 className="card-title">Sign up</h5>
      <div className="mb-3">
        <label htmlFor="exampleFormControlInput1" className="form-label">
          {nameLabel}
        </label>
        <input
          className="form-control"
          type="text"
          aria-label=".form-control-lg example"
          onChange={onChangeCredentials}
          name="name"
        />
      </div>

      <div className="mb-3">
        <label htmlFor="exampleFormControlInput1" className="form-label">
          Email
        </label>
        <input
          className="form-control"
          type="email"
          aria-label=".form-control-lg example"
          onChange={onChangeCredentials}
          name="email"
        />
      </div>

      <div className="mb-3">
        <label htmlFor="exampleFormControlInput1" className="form-label">
          Password
        </label>
        <input
          className="form-control"
          type="password"
          aria-label=".form-control-lg example"
          name="password"
          onChange={onChangeCredentials}
          onKeyDown={onKeyDown}
          onKeyUp={onKeyUp}
          id="password-0"
          value={credentials.password}
        />
      </div>
      <div className="mb-3">
        <label htmlFor="exampleFormControlInput1" className="form-label">
          Confirm Password
        </label>
        <input
          className="form-control"
          type="password"
          aria-label=".form-control-lg example"
          name="confrimPassword"
          onChange={onChangeCredentials}
          value={credentials.confrimPassword}
          onKeyDown={onKeyDown}
          onKeyUp={onKeyUp}
          id="password-1"
        />
      </div>

      <div className="form-check mb-3">
        <input
          className="form-check-input"
          type="checkbox"
          name="2FA"
          id="flexCheckDefault"
          checked={is2FAenabled}
          onChange={onChange2FAconcent}
        />
        <label className="form-check-label" htmlFor="flexCheckDefault">
          Enable keystroke dynamics 2FA
        </label>
      </div>
      {is2FAenabled && (
        <div className="mb-3">
          <label htmlFor="exampleFormControlInput1" className="form-label">
            Enter your password again
          </label>
          <input
            className="form-control"
            type="password"
            aria-label=".form-control-lg example"
            name="twoFAPassword"
            onChange={onChangeCredentials}
            value={credentials.twoFAPassword}
            onKeyDown={onKeyDown}
            onKeyUp={onKeyUp}
            id="password-2"
          />
        </div>
      )}
      <button
        type="button"
        className="btn btn-primary"
        disabled={disableSubmit}
        onClick={onSumit}
      >
        Sign up
      </button>
    </>
  );
};

export default SignUp;
